@@ -17,16 +17,25 @@
 #  limitations under the License.
 
 from .ipu_configuration import IPUConfig
-from .models.bart import PipelinedBartForConditionalGeneration
+from .models.bart import PipelinedBartForConditionalGeneration, PipelinedBartForSequenceClassification
 from .models.bert import (
+    PipelinedBertForMaskedLM,
     PipelinedBertForMultipleChoice,
     PipelinedBertForPreTraining,
     PipelinedBertForQuestionAnswering,
     PipelinedBertForSequenceClassification,
     PipelinedBertForTokenClassification,
 )
+from .models.convnext import PipelinedConvNextForImageClassification
+from .models.gpt2 import (
+    PipelinedGPT2ForSequenceClassification,
+    PipelinedGPT2ForTokenClassification,
+    PipelinedGPT2LMHeadModel,
+)
+from .models.hubert import PipelinedHubertForSequenceClassification
 from .models.lxmert import PipelinedLxmertForQuestionAnswering
 from .models.roberta import (
+    PipelinedRobertaForMaskedLM,
     PipelinedRobertaForMultipleChoice,
     PipelinedRobertaForQuestionAnswering,
     PipelinedRobertaForSequenceClassification,
@@ -34,7 +43,9 @@ from .models.roberta import (
 )
 from .models.t5 import PipelinedT5ForConditionalGeneration
 from .models.vit import PipelinedViTForImageClassification
-from .trainer import IPUTrainer
+from .models.wav2vec2 import PipelinedWav2Vec2ForPreTraining
+from .trainer import IPUTrainer, IPUTrainerState
 from .trainer_seq2seq import IPUSeq2SeqTrainer
 from .training_args import IPUTrainingArguments
 from .training_args_seq2seq import IPUSeq2SeqTrainingArguments
+from .version import __version__
