@@ -20,12 +20,17 @@ import torch.nn as nn
 
 import poptorch
 from optimum.utils import logging
-from transformers import BartForConditionalGeneration, BartModel
-from transformers.modeling_outputs import BaseModelOutput, Seq2SeqModelOutput
+from transformers import BartForConditionalGeneration, BartForSequenceClassification, BartModel
+from transformers.modeling_outputs import (
+    BaseModelOutput,
+    BaseModelOutputWithPastAndCrossAttentions,
+    Seq2SeqModelOutput,
+)
 from transformers.models.bart.modeling_bart import BartAttention, BartDecoder, BartEncoder, shift_tokens_right
 
 from ...generation_utils import IPUGenerationMixin
 from ...modeling_utils import (
+    GenerationMethodsMixin,
     PipelineMixin,
     SerializedLinear,
     SharedEmbedding,
@@ -199,7 +204,7 @@ class _BartEncoderWithCustomExpandMask(BartEncoder):
         inputs_embeds=None,
         output_attentions=None,
         output_hidden_states=None,
-        return_dict=None,
+        return_dict=False,
     ):
         output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
         output_hidden_states = (
@@ -480,15 +485,11 @@ class _BartModelWithSharedEmbedding(BartModel):
                 logger.warning("encoder and decoder embeddings computation is already shared")
             else:
                 self.shared = SharedEmbedding(self.shared)
-                self.encoder.embed_tokens = None
-                self.decoder.embed_tokens = None
         else:
             if isinstance(self.shared, nn.Embedding):
                 logger.warning("encoder and decoder embeddings computation is not shared")
             else:
                 self.shared = self.shared.shared
-                self.encoder.embed_tokens = self.shared
-                self.decoder.embed_tokens = self.shared
 
     def change_bart_encoder_and_decoder_classes(self, restore: bool):
         """Changes the encoder and decoder classes to update their forward pass so that they use our custom versions of
@@ -552,19 +553,18 @@ class _BartModelWithSharedEmbedding(BartModel):
         use_cache = use_cache if use_cache is not None else self.config.use_cache
         return_dict = return_dict if return_dict is not None else self.config.use_return_dict
 
-        if self.is_encoder_and_decoder_embeddings_computation_shared:
-            inputs_embeds, decoder_inputs_embeds = self.shared(
-                input_ids=input_ids,
-                decoder_input_ids=decoder_input_ids,
-                encoder_embed_scale=self.encoder.embed_scale,
-                decoder_embed_scale=self.decoder.embed_scale,
-            )
-            if inputs_embeds is not None:
-                input_ids = None
-            if decoder_inputs_embeds is not None:
-                decoder_input_ids = None
-
         if encoder_outputs is None:
+            if self.is_encoder_and_decoder_embeddings_computation_shared:
+                inputs_embeds, decoder_inputs_embeds = self.shared(
+                    input_ids=input_ids,
+                    decoder_input_ids=decoder_input_ids,
+                    encoder_embed_scale=self.encoder.embed_scale,
+                    decoder_embed_scale=self.decoder.embed_scale,
+                )
+                if inputs_embeds is not None:
+                    input_ids = None
+                if decoder_inputs_embeds is not None:
+                    decoder_input_ids = None
             encoder_outputs = self.encoder(
                 input_ids=input_ids,
                 attention_mask=attention_mask,
@@ -614,7 +614,9 @@ class _BartModelWithSharedEmbedding(BartModel):
 
 
 @register(BartForConditionalGeneration)
-class PipelinedBartForConditionalGeneration(IPUGenerationMixin, BartForConditionalGeneration, PipelineMixin):
+class PipelinedBartForConditionalGeneration(
+    GenerationMethodsMixin, BartForConditionalGeneration, PipelineMixin, IPUGenerationMixin
+):
     def parallelize(self):
         """
         Transform the model to run in an IPU pipeline.
@@ -696,11 +698,24 @@ class PipelinedBartForConditionalGeneration(IPUGenerationMixin, BartForCondition
         self.model.change_bart_encoder_and_decoder_classes(True)
         self.model.change_bart_attention_class(True)
         self.model.__class__ = BartModel
+
+        if self.ipu_config.embedding_serialization_factor > 1:
+            old_lm_head = nn.Linear(
+                self.config.d_model,
+                self.model.shared.num_embeddings,
+                bias=False,
+            )
+            old_lm_head.load_state_dict(self.lm_head.state_dict())
+            self.lm_head = old_lm_head
+            self.tie_weights()
+
         return self
 
     def train(self, mode: bool = True) -> "PipelinedBartForConditionalGeneration":
         mod = super(BartForConditionalGeneration, self).train(mode=mode)
-        mod.forward = mod._forward_for_train if mode else mod._forward_for_generate
+        # TODO: enable that once generation is supported.
+        # mod.forward = mod._forward_for_train if mode else mod._forward_for_generate
+        mod.forward = mod._forward_for_train
         return mod
 
     def _forward_for_train(self, input_ids, attention_mask, decoder_input_ids, labels=None):
@@ -713,15 +728,145 @@ class PipelinedBartForConditionalGeneration(IPUGenerationMixin, BartForCondition
             return_dict=False,
         )
         # Only returning the loss to make the communication between the host and the device faster.
-        return outputs[0]
+        return outputs[0:1]
 
-    def _forward_for_generate(self, encoder_outputs, decoder_input_ids, attention_mask):
-        return super().forward(
+    def _forward_for_generate(self, encoder_outputs, decoder_input_ids, attention_mask, labels=None):
+        outputs = super().forward(
             encoder_outputs=encoder_outputs,
             attention_mask=attention_mask,
             decoder_input_ids=decoder_input_ids,
             return_dict=False,
             use_cache=False,
+            labels=labels,
         )
+        # Only returning the loss (if labels is provided) and the logits.
+        if labels is None:
+            return outputs[:1]
+        return outputs[:2]
 
     forward = _forward_for_train
+
+
+@register(BartForSequenceClassification)
+class PipelinedBartForSequenceClassification(BartForSequenceClassification, PipelineMixin):
+    def parallelize(self):
+        """
+        Transform the model to run in an IPU pipeline.
+        - Adds pipeline stages to the model
+        - Adds recomputation checkpoints
+
+        Recommended usage:
+        ```
+        model = PipelinedBartForSequenceClassification(config).parallelize().half()
+        ```
+        """
+        super().parallelize()
+
+        self.model.__class__ = _BartModelWithSharedEmbedding
+        self.model.encoder_and_decoder_embeddings_computation(True)
+        self.model.change_bart_encoder_and_decoder_classes(False)
+        self.model.change_bart_attention_class(False)
+
+        logger.info("-------------------- Device Allocation --------------------")
+        logger.info("Embedding --> IPU 0")
+        self.model.shared = poptorch.BeginBlock(self.model.shared, "Embedding", ipu_id=0)
+        self.model.encoder.embed_positions = poptorch.BeginBlock(
+            self.model.encoder.embed_positions, "Embedding", ipu_id=0
+        )
+        self.model.encoder.layernorm_embedding = poptorch.BeginBlock(
+            self.model.encoder.layernorm_embedding, "Embedding", ipu_id=0
+        )
+
+        layer_ipu = get_layer_ipu(self.ipu_config.layers_per_ipu)
+        for index, layer in enumerate(self.model.encoder.layers):
+            ipu = layer_ipu[index]
+            if self.ipu_config.recompute_checkpoint_every_layer and index != self.config.num_hidden_layers - 1:
+                recomputation_checkpoint(layer)
+            self.model.encoder.layers[index] = poptorch.BeginBlock(layer, f"Encoder{index}", ipu_id=ipu)
+            logger.info(f"Encoder {index:<2} --> IPU {ipu}")
+
+        self.model.decoder.embed_positions = poptorch.BeginBlock(
+            self.model.decoder.embed_positions, "Embedding", ipu_id=0
+        )
+        self.model.decoder.layernorm_embedding = poptorch.BeginBlock(
+            self.model.decoder.layernorm_embedding, "Embedding", ipu_id=0
+        )
+        shift = len(self.model.encoder.layers)
+        for index, layer in enumerate(self.model.decoder.layers):
+            ipu = layer_ipu[index + shift]
+            if self.ipu_config.recompute_checkpoint_every_layer and index != self.config.num_hidden_layers - 1:
+                recomputation_checkpoint(layer)
+            self.model.decoder.layers[index] = poptorch.BeginBlock(layer, f"Decoder{index}", ipu_id=ipu)
+            logger.info(f"Decoder {index:<2} --> IPU {ipu}")
+
+        last_ipu = len(self.ipu_config.layers_per_ipu) - 1
+        logger.info(f"Classification Head Output --> IPU {last_ipu}")
+        self.classification_head = poptorch.BeginBlock(
+            self.classification_head, "Classification Head Output", ipu_id=last_ipu
+        )
+        logger.info("-----------------------------------------------------------")
+        return self
+
+    def deparallelize(self):
+        """
+        Undo the changes to the model done by `parallelize`.
+        You should call this before doing `save_pretrained` so that the `model.state_dict` is
+        fully compatible with `transformers.BartForSequenceClassification`.
+        """
+        super().deparallelize()
+
+        self.model.encoder_and_decoder_embeddings_computation(False)
+        self.model.change_bart_encoder_and_decoder_classes(True)
+        self.model.change_bart_attention_class(True)
+        self.model.__class__ = BartModel
+
+        return self
+
+    def forward(self, input_ids, attention_mask, labels=None):
+        outputs = self.model(
+            input_ids,
+            attention_mask=attention_mask,
+            use_cache=False,
+            return_dict=False,
+        )
+
+        hidden_states = outputs[0]  # last hidden state
+        B, L, E = hidden_states.shape
+
+        eos_mask = torch.eq(input_ids, self.config.eos_token_id)
+        # Static tensor shape version of hidden_states[eos_mask, :]
+        eos_indices = eos_mask * torch.arange(L).unsqueeze(0)
+        last_eos_index, _ = torch.max(eos_indices, dim=1)
+        # torch.index_select requires a 1D tensor of indices
+        last_eos_index += torch.arange(B) * L
+        hidden_states = hidden_states.view(B * L, E)
+        sentence_representation = torch.index_select(hidden_states, 0, last_eos_index)
+
+        logits = self.classification_head(sentence_representation)
+
+        loss = None
+        if labels is not None:
+            if self.config.problem_type is None:
+                if self.config.num_labels == 1:
+                    self.config.problem_type = "regression"
+                elif self.config.num_labels > 1 and (labels.dtype == torch.long or labels.dtype == torch.int):
+                    self.config.problem_type = "single_label_classification"
+                else:
+                    self.config.problem_type = "multi_label_classification"
+
+            if self.config.problem_type == "regression":
+                loss_fct = nn.MSELoss()
+                if self.config.num_labels == 1:
+                    loss = loss_fct(logits.squeeze(), labels.squeeze())
+                else:
+                    loss = loss_fct(logits, labels)
+            elif self.config.problem_type == "single_label_classification":
+                loss_fct = nn.CrossEntropyLoss()
+                loss = loss_fct(logits.view(-1, self.config.num_labels), labels.view(-1))
+            elif self.config.problem_type == "multi_label_classification":
+                loss_fct = nn.BCEWithLogitsLoss()
+                loss = loss_fct(logits, labels)
+
+        if labels is None:
+            return (logits,)
+        return loss, logits
