@@ -23,6 +23,7 @@ import shutil
 import sys
 import time
 import warnings
+from dataclasses import dataclass
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple, Union
 
@@ -121,6 +122,11 @@ DEFAULT_CALLBACKS = [DefaultFlowCallback]
 DEFAULT_PROGRESS_CALLBACK = ProgressCallback
 
 
+@dataclass
+class IPUTrainerState(TrainerState):
+    start_time: float = -1.0
+
+
 class IPUTrainer:
 
     from transformers.trainer_pt_utils import log_metrics, metrics_format, save_metrics, save_state
@@ -191,8 +197,8 @@ class IPUTrainer:
             logger.info(f"Overriding IPU config: {args.ipu_config_overrides}")
             self.ipu_config.update_from_string(args.ipu_config_overrides)
         self.ipu_config.seed = self.args.seed
-        self.opts = self.ipu_config.to_options()
-        self.eval_opts = self.ipu_config.to_options(for_inference=True)
+        self.opts = self.ipu_config.to_options(compile_only=args.compile_only)
+        self.eval_opts = self.ipu_config.to_options(for_inference=True, compile_only=args.compile_only)
 
         self.model = to_pipelined(model, self.ipu_config, force=force_to_pipelined)
         self.model.parallelize()
@@ -202,6 +208,7 @@ class IPUTrainer:
         if not self.args.fp32:
             self.model = self.model.half()
 
+        self.model_wrapped = self.model
         self.training_model = None
         self.inference_model = None
 
@@ -231,6 +238,9 @@ class IPUTrainer:
         if self.args.push_to_hub:
             self.init_git_repo()
 
+        if self.args.should_save:
+            os.makedirs(self.args.output_dir, exist_ok=True)
+
         if not callable(self.data_collator) and callable(getattr(self.data_collator, "collate_batch", None)):
             raise ValueError("The `data_collator` should be a simple callable (function, class with `__call__`).")
 
@@ -248,7 +258,7 @@ class IPUTrainer:
         else:
             self.label_smoother = None
 
-        self.state = TrainerState()
+        self.state = IPUTrainerState()
         self.control = TrainerControl()
         # Internal variable to count flos in each process, will be accumulated in `self.state.total_flos` then
         # returned to 0 every time flos need to be logged
@@ -266,6 +276,20 @@ class IPUTrainer:
         # very last
         self._memory_tracker.stop_and_update_metrics()
 
+        # If compile-only then compile and exit
+        if args.compile_only:
+            logger.info("Called with compile_only=True. Compiling models then exiting.")
+            if args.do_train:
+                train_dl = self.get_train_dataloader()
+                model = self._wrap_model(self.model_wrapped)
+                self._compile_model(model, next(iter(train_dl)), log=True)
+            if args.do_eval:
+                # Same thing with _wrap_and_compile_for_evaluation
+                eval_dl = self.get_eval_dataloader()
+                model = self._wrap_and_compile_model_for_evaluation(eval_dl, False)
+            logger.info("Exiting after compiling models with compile_only=True")
+            sys.exit(0)
+
     def _pytorch_optimizer_to_poptorch(
         self,
         optimizer: optim.Optimizer,
@@ -828,7 +852,6 @@ class IPUTrainer:
         total_train_batch_size = args.per_device_train_batch_size * self.ipu_config.batch_size_factor()
         if train_dataset_is_sized:
             # No need to divide by the number of gradient accumulation steps as poptorch already accounts for that.
-            # num_update_steps_per_epoch = len(train_dataloader) // args.gradient_accumulation_steps
             num_update_steps_per_epoch = len(train_dataloader)
             num_update_steps_per_epoch = max(num_update_steps_per_epoch, 1)
             if args.max_steps > 0:
@@ -836,14 +859,12 @@ class IPUTrainer:
                 num_train_epochs = args.max_steps // num_update_steps_per_epoch + int(
                     args.max_steps % num_update_steps_per_epoch > 0
                 )
-
-                # May be slightly incorrect if the last batch in the training datalaoder has a smaller size but it's
-                # the best we can do.
-                num_train_samples = args.max_steps * total_train_batch_size
             else:
                 max_steps = math.ceil(args.num_train_epochs * num_update_steps_per_epoch)
                 num_train_epochs = math.ceil(args.num_train_epochs)
-                num_train_samples = len(self.train_dataset) * args.num_train_epochs
+            # May be slightly incorrect if the last batch in the training dataloader has a smaller size but it's
+            # the best we can do.
+            num_train_samples = max_steps * total_train_batch_size
         else:
             # see __init__. max_steps is set when the dataset has no __len__
             max_steps = args.max_steps
@@ -857,13 +878,25 @@ class IPUTrainer:
 
         self.create_optimizer_and_scheduler(num_training_steps=max_steps)
 
-        self.state = TrainerState()
+        self.state = IPUTrainerState()
         if trial is not None:
             raise ValueError("Hyperparameter tuning is not supported by the IPUTrainer.")
             trial = None
         self.state.is_hyper_param_search = trial is not None
 
-        model = self._wrap_model(self.model)
+        # Activate gradient checkpointing if needed
+        if args.gradient_checkpointing:
+            self.model.gradient_checkpointing_enable()
+
+        model = self._wrap_model(self.model_wrapped)
+
+        # for the rest of this function `model` is the outside model, whether it was wrapped or not
+        if model is not self.model:
+            self.model_wrapped = model
+
+        # TODO: handle optimizer and scheduler creation
+        # if delay_optimizer_creation:
+        #     self.create_optimizer_and_scheduler(num_training_steps=max_steps)
 
         # Check if saved optimizer or scheduler states exist
         self._load_optimizer_and_scheduler(resume_from_checkpoint)
@@ -895,7 +928,10 @@ class IPUTrainer:
         if resume_from_checkpoint is not None and os.path.isfile(
             os.path.join(resume_from_checkpoint, TRAINER_STATE_NAME)
         ):
-            self.state = TrainerState.load_from_json(os.path.join(resume_from_checkpoint, TRAINER_STATE_NAME))
+            self.state = IPUTrainerState.load_from_json(os.path.join(resume_from_checkpoint, TRAINER_STATE_NAME))
+            if self.state.start_time < 0:
+                self.state.start_time = start_time
+            start_time = self.state.start_time
             epochs_trained = self.state.global_step // num_update_steps_per_epoch
             if not args.ignore_data_skip:
                 steps_trained_in_current_epoch = self.state.global_step % (num_update_steps_per_epoch)
@@ -928,6 +964,7 @@ class IPUTrainer:
         # to set this after the load.
         self.state.max_steps = max_steps
         self.state.num_train_epochs = num_train_epochs
+        self.state.start_time = start_time
 
         # tr_loss is a tensor to avoid synchronization of TPUs through .item()
         tr_loss = torch.tensor(0.0).to(args.device)
@@ -1064,7 +1101,10 @@ class IPUTrainer:
             self.model.half()
 
         if len(load_result.missing_keys) != 0:
-            logger.warn(f"There were missing keys in the checkpoint model loaded: {load_result.missing_keys}.")
+            if self.model._keys_to_ignore_on_save is not None and set(load_result.missing_keys) != set(
+                self.model._keys_to_ignore_on_save
+            ):
+                logger.warn(f"There were missing keys in the checkpoint model loaded: {load_result.missing_keys}.")
         if len(load_result.unexpected_keys) != 0:
             logger.warn(f"There were unexpected keys in the checkpoint model loaded: {load_result.unexpected_keys}.")
 
@@ -1324,9 +1364,11 @@ class IPUTrainer:
                 state_dict = self.model.state_dict()
             torch.save(state_dict, os.path.join(output_dir, WEIGHTS_NAME))
         else:
+            rng_state = torch.random.get_rng_state()
             self.model.deparallelize()
             self.model.save_pretrained(output_dir, state_dict=state_dict)
             self.model.parallelize()
+            torch.random.set_rng_state(rng_state)
 
         if self.tokenizer is not None:
             self.tokenizer.save_pretrained(output_dir)
@@ -1434,6 +1476,15 @@ class IPUTrainer:
         self._memory_tracker.start()
 
         eval_dataloader = self.get_eval_dataloader(eval_dataset)
+
+        prediction_loss_only = True if self.compute_metrics is None else None
+        if prediction_loss_only is None:
+            prediction_loss_only = self.args.prediction_loss_only
+
+        # Running this here (even though it is being recalled in self.evaluation_loop to make compilation happen here.
+        # That way, compilation will not mess inference speed metrics.
+        _ = self._wrap_and_compile_model_for_evaluation(eval_dataloader, prediction_loss_only)
+
         start_time = time.time()
 
         output = self.evaluation_loop(
@@ -1441,7 +1492,7 @@ class IPUTrainer:
             description="Evaluation",
             # No point gathering the predictions if there are no metrics, otherwise we defer to
             # self.args.prediction_loss_only
-            prediction_loss_only=True if self.compute_metrics is None else None,
+            prediction_loss_only=prediction_loss_only,
             ignore_keys=ignore_keys,
             metric_key_prefix=metric_key_prefix,
         )
@@ -1520,6 +1571,11 @@ class IPUTrainer:
 
         return PredictionOutput(predictions=output.predictions, label_ids=output.label_ids, metrics=output.metrics)
 
+    def _wrap_and_compile_model_for_evaluation(self, dataloader, prediction_loss_only):
+        model = self._wrap_model(self.model, training=False)
+        self._compile_model(model, next(iter(dataloader)), log=True)
+        return model
+
     def evaluation_loop(
         self,
         dataloader: poptorch.DataLoader,
@@ -1537,8 +1593,7 @@ class IPUTrainer:
             prediction_loss_only if prediction_loss_only is not None else self.args.prediction_loss_only
         )
 
-        model = self._wrap_model(self.model, training=False)
-        self._compile_model(model, next(iter(dataloader)), log=True)
+        model = self._wrap_and_compile_model_for_evaluation(dataloader, prediction_loss_only)
 
         batch_size = dataloader.batch_size
 
@@ -1549,8 +1604,6 @@ class IPUTrainer:
             logger.info("  Num examples: Unknown")
         logger.info(f"  Batch size = {batch_size}")
 
-        model.eval()
-
         self.callback_handler.eval_dataloader = dataloader
         # Do this before wrapping.
         eval_dataset = dataloader.dataset
@@ -1592,10 +1645,10 @@ class IPUTrainer:
             # Update containers on host
             if loss is not None:
                 loss = loss.mean(dim=0, keepdim=True)
-
                 # If only one IPU is used, loss is a zero dimensional tensor, we unsqueeze to be able to concatenate.
                 if loss.dim() == 0:
                     loss = loss.unsqueeze(0)
+
                 losses_host = loss if losses_host is None else torch.cat((losses_host, loss), dim=0)
             if logits is not None:
                 if self.preprocess_logits_for_metrics is not None:
@@ -1621,14 +1674,7 @@ class IPUTrainer:
             all_labels = labels if all_labels is None else nested_concat(all_labels, labels, padding_index=-100)
 
         # Number of samples
-        if not isinstance(eval_dataset, IterableDataset):
-            num_samples = len(eval_dataset)
-        # The instance check is weird and does not actually check for the type, but whether the dataset has the right
-        # methods. Therefore we need to make sure it also has the attribute.
-        elif isinstance(eval_dataset, IterableDatasetShard) and hasattr(eval_dataset, "num_examples"):
-            num_samples = eval_dataset.num_examples
-        else:
-            num_samples = observed_num_examples
+        num_samples = observed_num_examples
 
         # Number of losses has been rounded to a multiple of batch_size and in a distributed training, the number of
         # samplers has been rounded to a multiple of batch_size, so we truncate.
@@ -1657,7 +1703,12 @@ class IPUTrainer:
                 metrics[f"{metric_key_prefix}_{key}"] = metrics.pop(key)
 
         # Detaching model from device to let the training model attach itself
-        model.detachFromDevice()
+        if isinstance(model, PoplarExecutor):
+            model.detachFromDevice()
+        else:
+            if not self.ipu_config.execute_encoder_on_cpu_for_generation:
+                model.get_encoder().detachFromDevice()
+            model.poptorch_model.detachFromDevice()
 
         return EvalLoopOutput(predictions=all_preds, label_ids=all_labels, metrics=metrics, num_samples=num_samples)
 
